.bslib-value-box{--bslib-value-box-separator-color: rgba(var(--bs-white-rgb, 255,255,255), 0.175)}.bslib-value-box .value-box-grid{grid-template-columns:var(--bslib-value-box-widths)}.bslib-value-box .value-box-showcase{align-items:center;justify-content:center;margin-top:auto;margin-bottom:auto;padding:1rem;overflow:hidden;max-height:var(--bslib-value-box-max-height)}.bslib-value-box .value-box-showcase .bi,.bslib-value-box .value-box-showcase .fa{opacity:.85;min-width:50px;max-width:125%}.bslib-value-box .value-box-showcase .bi{font-size:5rem}.bslib-value-box .value-box-showcase .fa{font-size:4rem}.bslib-value-box .value-box-showcase.showcase-top-right{align-items:end;padding-left:0;padding-bottom:0}.bslib-value-box .value-box-area{justify-content:center;padding:1.5rem 1rem;font-size:.9rem;font-weight:500}.bslib-value-box .value-box-area *{color:inherit;margin-bottom:0;margin-top:0}.bslib-value-box .value-box-area>:first-child{font-size:1rem;margin-top:0;margin-bottom:.5rem;font-weight:500;line-height:1.2;color:var(--bs-heading-color);color:inherit}.bslib-value-box .value-box-area>:first-child::after{content:'\00a0 '}.bslib-value-box .value-box-area>:nth-child(2){font-size:calc(1.325rem + .9vw);margin-top:0;margin-bottom:.5rem;font-weight:500;line-height:1.2;color:var(--bs-heading-color);color:inherit}@media (min-width: 1200px){.bslib-value-box .value-box-area>:nth-child(2){font-size:2rem}}.bslib-value-box .value-box-area>:nth-child(2)::after{content:'\00a0 '}.bslib-value-box .value-box-area.border-start{border-color:var(--bslib-value-box-separator-color) !important}.bslib-value-box[data-full-screen="true"] .value-box-grid{grid-template-columns:var(--bslib-value-box-widths-full-screen)}.bslib-value-box[data-full-screen="true"] .value-box-showcase{max-height:var(--bslib-value-box-max-height-full-screen)}.bslib-value-box:not([data-full-screen="true"]) .value-box-showcase.showcase-top-right{margin-top:0}@media (max-width: 575.98px){.bslib-value-box .value-box-grid{grid-template-columns:var(--bslib-value-box-widths) !important}}
